# shellcheck disable=SC2148
VENDOR="${VENDOR:-solarwinds}"
# shellcheck disable=SC2034
APPOPTICS_URL=${APPOPTICS_URL:-"https://api.appoptics.com/v1/measurements"}
# shellcheck disable=SC2034
APPOPTICS_REPORT_URL="https://api.appoptics.com/v1/agent/report"
# shellcheck disable=SC2034
LOGGLY_TOKEN=${LOGGLY_TOKEN:-"SOLARWINDS_TOKEN"}
DEFAULT_INSTALL_DIR="/opt/SolarWinds/Snap"
INSTALL_DIR=${INSTALL_DIR:-${DEFAULT_INSTALL_DIR}}
# shellcheck disable=SC2034
AGENT_CONF="${INSTALL_DIR}/etc/config.yaml"
# shellcheck disable=SC2034
PLUGINS_CONFIG_DIR="${INSTALL_DIR}/etc/plugins.d"
# shellcheck disable=SC2034
LOGS_PLUGIN_CONFIG="${PLUGINS_CONFIG_DIR}/logs.yaml"
# shellcheck disable=SC2034
LOGS_PLUGIN_CONFIG_V2="${PLUGINS_CONFIG_DIR}/logs-v2.yaml"
# shellcheck disable=SC2034
TASKS_CONFIG_DIR="${INSTALL_DIR}/etc/tasks.d"
# shellcheck disable=SC2034
AUTOLOAD_DIR="${INSTALL_DIR}/autoload"
# shellcheck disable=SC2034
TASKS_AUTOLOAD_CONFIG_DIR="${INSTALL_DIR}/etc/tasks-autoload.d"
# shellcheck disable=SC2034
LOGS_PLUGIN_CONFIG_EXAMPLE="${LOGS_PLUGIN_CONFIG}.example"
# shellcheck disable=SC2034
PUBLISHER_AO_CONFIG="${PLUGINS_CONFIG_DIR}/publisher-appoptics.yaml"
# shellcheck disable=SC2034
PUBLISHER_PROCESSES_CONFIG="${PLUGINS_CONFIG_DIR}/publisher-processes.yaml"
# shellcheck disable=SC2034
PUBLISHER_LOGS_CONFIG="${PLUGINS_CONFIG_DIR}/publisher-logs.yaml"

# shellcheck disable=SC2034
FILES_WITH_TOKEN="${PUBLISHER_AO_CONFIG} ${PUBLISHER_PROCESSES_CONFIG}"

# shellcheck disable=SC2034
AGENT_NAME="swisnapd"
# shellcheck disable=SC2034
DAEMON_NAME="swisnapd"
# shellcheck disable=SC2034
AGENT_DISPLAY_NAME="SolarWinds Snap Agent"

# shellcheck disable=SC2034
AGENT_TMP_DIR="/tmp/SolarWinds/Snap"
# shellcheck disable=SC2034
AGENT_LOG_DIR="/var/log/SolarWinds/Snap"
# shellcheck disable=SC2034
AGENT_RUN_DIR="/opt/SolarWinds/Snap/run"
# shellcheck disable=SC2034
AGENT_CACHE_DIR="/opt/SolarWinds/Snap/cache"
# shellcheck disable=SC2034
PID_DIR="/var/run/${VENDOR}"
# shellcheck disable=SC2034
SYMLINK_DIR=/usr/local/bin
# shellcheck disable=SC2034
LOG_ROTATION_CONF_DIR=/etc/logrotate.d

# shellcheck disable=SC2034
SYSTEMCTL_BIN=$(command -v systemctl 2>/dev/null)
# shellcheck disable=SC2034
CHKCONFIG_BIN=$(command -v chkconfig 2>/dev/null)
# shellcheck disable=SC2034
SERVICE_BIN=$(command -v service 2>/dev/null)
# shellcheck disable=SC2034
DPKG_QUERY_BIN=$(command -v dpkg-query 2>/dev/null)
# shellcheck disable=SC2034
LOGROTATE_BIN=$(command -v logrotate 2>/dev/null)
# shellcheck disable=SC2034
SETCAP_BIN=$(command -v setcap 2>/dev/null)
# shellcheck disable=SC2034
UPGRADE_SAVE_TOKEN_FILE="${INSTALL_DIR}/etc/upgrade.token.backup"
# shellcheck disable=SC2034
RM_CMD=/bin/rm

# Settings for the user used to run the daemon
# shellcheck disable=SC2034
AGENT_USERNAME=${VENDOR}
# shellcheck disable=SC2034
AGENT_GROUPNAME=${VENDOR}

# shellcheck disable=SC2034
SCRIPT_PARAMS="$@"

# truth values
__IS_TRUE=1
__IS_FALSE=0

# When set to true, install the package but don't start the service
AGENT_INSTALL_ONLY=${AGENT_INSTALL_ONLY:-${APPOPTICS_INSTALL_ONLY:-${__IS_FALSE}}}

# Contains the current installation step of the post installer
# shellcheck disable=SC2034
__step=1

_ECHO_DEBUG=${_ECHO_DEBUG:-${__IS_FALSE}}
_SD_NO_COLOR=${_SD_NO_COLOR:-${__IS_TRUE}}

# shellcheck disable=SC2034
ROOT_DIR=$(pwd)

# Return error codes
readonly SUCCESS=0
readonly FAILURE=1
readonly FAILURE_SERVICE_NOT_RUNNING=2
readonly FAILURE_NOT_ENOUGH_RIGHTS=3
readonly FAILURE_UNABLE_TO_EXTRACT=4
readonly FAILURE_MISSING_EXECUTABLE=5
readonly FAILURE_CREATE_AGENT_GROUP=6
readonly FAILURE_CREATE_AGENT_USER=7
readonly FAILURE_SET_ACCESS_RIGHTS_AGENT_USER=8
_COLORS=${BS_COLORS:-$(tput colors 2>/dev/null || echo 0)}
detect_color_support() {
    # shellcheck disable=SC2181
    if [ $? -eq 0 ] && [ "$_COLORS" -gt 2 ] && [ "$_SD_NO_COLOR" -eq "$__IS_FALSE" ]; then
        RC="\033[1;31m"
        GC="\033[1;32m"
        BC="\033[1;34m"
        YC="\033[1;33m"
        EC="\033[0m"
    else
        RC=""
        GC=""
        BC=""
        YC=""
        EC=""
    fi
}
detect_color_support

# Override built-in echo for consistency. Has newline at end
echo() (
    fmt=%s end=\\n IFS=" "

    while [ $# -gt 1 ]; do
        case "$1" in
        [!-]* | -*[!ne]*) break ;;
        *ne* | *en*) fmt=%b end= ;;
        *n*) end= ;;
        *e*) fmt=%b ;;
        esac
        shift
    done

    # shellcheck disable=SC2059
    printf "$fmt$end" "$*"
)

echored() {
    printf "${RC}%s${EC}\n" "$*"
}

echogreen() {
    printf "${GC}%s${EC}\n" "$*"
}

echoblue() {
    printf "${BC}%s${EC}\n" "$*"
}

echoyellow() {
    printf "${YC}%s${EC}\n" "$*"
}
__increment_step() {
    __step=$((__step + 1))
}

exists() {
    command -v "$1" >/dev/null 2>&1
}

now() {
    date +'%Y-%m-%d-%H_%M_%S'
}

create_install_directories() {
    echodebug "Creating support directories ..."
    mkdir -p "${AGENT_TMP_DIR}"
    mkdir -p "${AGENT_LOG_DIR}"
    mkdir -p "${PID_DIR}"
    mkdir -p "${AGENT_RUN_DIR}"
    mkdir -p "${AGENT_CACHE_DIR}"
    echodebug "directories created."
}

get_package_version() {
    local version=
    if [ -x "${DPKG_QUERY_BIN}" ]; then
        version="$(dpkg-query -W solarwinds-snap-agent)"
    else
        version="$(rpm -q solarwinds-snap-agent)"
    fi
    echo "${version}"
}

set_swoken() {
    if [ -n "${SOLARWINDS_TOKEN}" ]; then
        SOLARWINDS_TOKEN=${SOLARWINDS_TOKEN}
    elif [ -n "${APPOPTICS_TOKEN}" ]; then
        SOLARWINDS_TOKEN=${APPOPTICS_TOKEN}
    else
        SOLARWINDS_TOKEN="SOLARWINDS_TOKEN"
    fi
}

insert_appoptics_url() {
    echodebug "Setting AppOptics endpoints in configuration files"
    sed -i "s,APPOPTICS_URL,\"${APPOPTICS_URL}\",g" "${AGENT_CONF}" "${PUBLISHER_AO_CONFIG}" "${PUBLISHER_PROCESSES_CONFIG}" "${PUBLISHER_LOGS_CONFIG}"
    sed -i "s,APPOPTICS_REPORT_URL,\"${APPOPTICS_REPORT_URL}\",g" "${AGENT_CONF}" "${PUBLISHER_AO_CONFIG}" "${PUBLISHER_PROCESSES_CONFIG}" "${PUBLISHER_LOGS_CONFIG}"
}

insert_loggly_token() {
    echodebug "Setting token in ${LOGS_PLUGIN_CONFIG} configuration file..."
    if [ ! -z "$1" ]; then
        sed -i "s/SOLARWINDS_TOKEN/${1}/g" "${LOGS_PLUGIN_CONFIG}" "${LOGS_PLUGIN_CONFIG_V2}" 2>/dev/null
    else
        echodebug "token not provided, please update log plugin configuration file after installation"
    fi
}

get_current_token() {
    grep '^[[:blank:]]*token' "${AGENT_CONF}" "${PUBLISHER_AO_CONFIG}" "${PUBLISHER_PROCESSES_CONFIG}" "${PUBLISHER_LOGS_CONFIG}" 2>/dev/null | tr -d '[:blank:]"' | tr -d "'" | cut -d ':' -f3 | head -1
}

replace_token() {
    echodebug "Setting token in configuration files"
    if [ ! -z "$1" ] && [ ! -z "$2" ]; then
        sed -i "s/${1}/${2}/g" "${AGENT_CONF}" "${PUBLISHER_AO_CONFIG}" "${PUBLISHER_PROCESSES_CONFIG}" "${PUBLISHER_LOGS_CONFIG}"
    else
        echodebug "token not provided, please update publisher plugin configuration files after installation"
    fi
}

remove_deprecated_configs() {
    if dpkg-maintscript-helper help > /dev/null 2>&1; then
        configs="cassandra.yaml.example \
            collectd_listener.yaml.example \
            consul.yaml.example \
            exec.yaml.example \
            fluentd.yaml.example \
            graphite_listener.yaml.example \
            influxdb_listener.yaml.example \
            nagios_exec.yaml.example \
            nginx.yaml.example \
            ntpq.yaml.example \
            phpfpm.yaml.example \
            postgresql.yaml.example \
            prometheus.yaml.example \
            redis.yaml.example \
            socket_listener.yaml.example \
            sqlserver.yaml.example \
            statsd.yaml.example \
            tomcat.yaml.example \
            varnish.yaml.example \
            zookeeper.yaml.example"

        for config in $configs; do
            dpkg-maintscript-helper rm_conffile "${PLUGINS_CONFIG_DIR}/${config}" 2.7.5.577 solarwinds-snap-agent -- "$1" "$SCRIPT_PARAMS"
        done

        tasks="task-logs-system.yaml.example \
            task-bridge-jolokia.yaml.example"

        for task in $tasks; do
            dpkg-maintscript-helper rm_conffile "${TASKS_AUTOLOAD_CONFIG_DIR}/${task}" 4.0.1.881 solarwinds-snap-agent -- "$1" "$SCRIPT_PARAMS"
        done
    fi
}
echoerror() {
    printf "${RC} * ERROR${EC}: %s\n" "$*" 1>&2
}

info() {
    printf "${GC} *  INFO${EC}: %s\n" "$*"
}

warning() {
    printf "${YC} *  WARN${EC}: %s\n" "$*"
}

echodebug() {
    if [ "$_ECHO_DEBUG" = "$__IS_TRUE" ]; then
        printf "${BC} * DEBUG${EC}: %s\n" "$*"
    fi
}

error() {
    error_code=${1}
    error_description=${2:-""}
    error_date_time="$(now)"
    echoerror "######################################################"
    echoerror "An error has occured during uninstallation of the agent."
    echoerror
    echoerror "Description: ${error_description}"
    echoerror "Error code : ${error_code}"
    echoerror "When       : ${error_date_time}"
    echoerror "######################################################"
}

failed() {
    error_code=${1}
    echo -e "${BC}===============${RC} FAILED ${BC}======================${EC}"
    echo -e "${BC}An error occurred during installation of ${RC}${AGENT_DISPLAY_NAME}${BC}.${EC}"
    echo -e "${BC}Please contact SolarWinds Technical Support at ${RC}technicalsupport@solarwinds.com${BC}.${EC}"
    echo -e "${BC}Error code: ${RC}${error_code}${BC}.${EC}"
    echo -e "${BC}========================================================${EC}"
}
# before-remove

remove_service() {
    info "(${__step}) Checking if system uses systemd or SysV(/etc/init.d) to manage the services."
    __increment_step

    if [ -x "${SYSTEMCTL_BIN}" ]; then
        # Stop service
        info "(${__step}) Stopping ${AGENT_DISPLAY_NAME} service ..."
        __increment_step
        ${SYSTEMCTL_BIN} stop "${AGENT_NAME}"

        # Disable service to be executed at boot time
        info "(${__step}) Disabling ${AGENT_DISPLAY_NAME} service to be loaded at boot time ..."
        __increment_step
        ${SYSTEMCTL_BIN} disable "${AGENT_NAME}"
    else
        # Stop service
        info "(${__step}) Stopping ${AGENT_DISPLAY_NAME} service ..."
        __increment_step
        ${SERVICE_BIN} "${AGENT_NAME}" stop

        # Disable service to be executed at boot time
        info "(${__step}) Disabling ${AGENT_DISPLAY_NAME} service to be loaded at boot time ..."
        __increment_step
        if [ -x "${CHKCONFIG_BIN}" ]; then
            ${CHKCONFIG_BIN} "${AGENT_NAME}" off
        else
            /usr/sbin/update-rc.d -f "${AGENT_NAME}" remove
        fi

        # Remove service
        echodebug "Remove configuration for ${AGENT_DISPLAY_NAME} service ..."
        ${RM_CMD} -f "/etc/init.d/${AGENT_NAME}"
    fi
}

remove_daemon_user() {
    info "(${__step}) Removing user and group settings ..."
    __increment_step

    if getent passwd "${AGENT_USERNAME}" >/dev/null; then
        info "Deleting ${AGENT_USERNAME} system user ..."
        userdel "${AGENT_USERNAME}"
        RETVAL=$?
        if [ $RETVAL -ne 0 ]; then
            warning "Unable to delete ${AGENT_USERNAME} system user. Error code: ${RETVAL}."
        else
            info "${AGENT_USERNAME} deleted."
        fi
    fi

    if getent group "${AGENT_GROUPNAME}" >/dev/null; then
        info "Deleting ${AGENT_GROUPNAME} system group ..."
        groupdel "${AGENT_GROUPNAME}"
        RETVAL=$?
        if [ $RETVAL -ne 0 ]; then
            warning "Unable to delete ${AGENT_GROUPNAME} system group. Error code: ${RETVAL}."
        else
            info "${AGENT_GROUPNAME} deleted."
        fi
    fi

    echodebug "user and group settings removed."
}

main() {
    echo -e " ${BC}======================${GC} ${AGENT_DISPLAY_NAME} Uninstall ${BC}=======================${EC}"
    echoblue "  This script will complete uninstallation of ${AGENT_DISPLAY_NAME}."

    remove_service
    remove_daemon_user

    echo -e "${BC} Uninstall of ${GC}${AGENT_DISPLAY_NAME}${BC} has completed.${EC}"

    echo -e " ${BC}======================${GC} ${AGENT_DISPLAY_NAME} Uninstall Completed ${BC}=======================${EC}"
}

main